import { Input } from '@angular/core';
/**
 * Base class for all Ionic components. Exposes some common functionality
 * that all Ionic components need, such as accessing underlying native elements and
 * sending/receiving app-level events.
 */
/** @hidden */
var Ion = (function () {
    function Ion(config, elementRef, renderer, componentName) {
        this._config = config;
        this._elementRef = elementRef;
        this._renderer = renderer;
        this._componentName = componentName;
        if (componentName) {
            this._setComponentName();
            this._setMode(config.get('mode'));
        }
    }
    Object.defineProperty(Ion.prototype, "color", {
        get: function () {
            return this._color;
        },
        set: /**
           * @input {string} The color to use from your Sass `$colors` map.
           * Default options are: `"primary"`, `"secondary"`, `"danger"`, `"light"`, and `"dark"`.
           * For more information, see [Theming your App](/docs/theming/theming-your-app).
           */
        function (val) {
            this._setColor(val);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Ion.prototype, "mode", {
        get: function () {
            return this._mode;
        },
        set: /**
           * @input {string} The mode determines which platform styles to use.
           * Possible values are: `"ios"`, `"md"`, or `"wp"`.
           * For more information, see [Platform Styles](/docs/theming/platform-specific-styles).
           */
        function (val) {
            this._setMode(val);
        },
        enumerable: true,
        configurable: true
    });
    /** @hidden */
    /** @hidden */
    Ion.prototype.setElementClass = /** @hidden */
    function (className, isAdd) {
        this._renderer.setElementClass(this._elementRef.nativeElement, className, isAdd);
    };
    /** @hidden */
    /** @hidden */
    Ion.prototype.setElementAttribute = /** @hidden */
    function (attributeName, attributeValue) {
        this._renderer.setElementAttribute(this._elementRef.nativeElement, attributeName, attributeValue);
    };
    /** @hidden */
    /** @hidden */
    Ion.prototype.setElementStyle = /** @hidden */
    function (property, value) {
        this._renderer.setElementStyle(this._elementRef.nativeElement, property, value);
    };
    /** @hidden */
    /** @hidden */
    Ion.prototype._setColor = /** @hidden */
    function (newColor, componentName) {
        if (componentName) {
            // This is needed for the item-radio
            this._componentName = componentName;
        }
        if (this._color) {
            this.setElementClass(this._componentName + "-" + this._mode + "-" + this._color, false);
        }
        if (newColor) {
            this.setElementClass(this._componentName + "-" + this._mode + "-" + newColor, true);
            this._color = newColor;
        }
    };
    /** @hidden */
    /** @hidden */
    Ion.prototype._setMode = /** @hidden */
    function (newMode) {
        if (this._mode) {
            this.setElementClass(this._componentName + "-" + this._mode, false);
        }
        if (newMode) {
            this.setElementClass(this._componentName + "-" + newMode, true);
            // Remove the color class associated with the previous mode,
            // change the mode, then add the new color class
            this._setColor(null);
            this._mode = newMode;
            this._setColor(this._color);
        }
    };
    /** @hidden */
    /** @hidden */
    Ion.prototype._setComponentName = /** @hidden */
    function () {
        this.setElementClass(this._componentName, true);
    };
    /** @hidden */
    /** @hidden */
    Ion.prototype.getElementRef = /** @hidden */
    function () {
        return this._elementRef;
    };
    /** @hidden */
    /** @hidden */
    Ion.prototype.getNativeElement = /** @hidden */
    function () {
        return this._elementRef.nativeElement;
    };
    Ion.propDecorators = {
        "color": [{ type: Input },],
        "mode": [{ type: Input },],
    };
    return Ion;
}());
export { Ion };
//# sourceMappingURL=ion.js.map